package ru.dbotthepony.mc.otm.datagen

import it.unimi.dsi.fastutil.objects.Object2ObjectArrayMap
import net.minecraft.data.DataGenerator
import net.minecraft.world.effect.MobEffect
import net.minecraft.world.entity.EntityType
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.enchantment.Enchantment
import net.minecraft.world.level.block.Block
import net.minecraftforge.common.data.LanguageProvider
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.android.AndroidFeatureType
import ru.dbotthepony.mc.otm.android.AndroidResearchType
import ru.dbotthepony.mc.otm.registry.ColoredDecorativeBlock

class MatteryLanguageProvider(private val gen: DataGenerator) {
	private inner class Slave(language: String) : LanguageProvider(gen, OverdriveThatMatters.MOD_ID, language) {
		init {
			gen.addProvider(this)
		}

		override fun addTranslations() {}
	}

	private val slaves = Object2ObjectArrayMap<String, Slave>()

	inner class Builder(language: String) {
		val slave: LanguageProvider = slaves.computeIfAbsent(language, ::Slave)

		fun add(key: String, value: String) = slave.add(key, value)
		fun add(key: Block, value: String) = slave.add(key, value)
		fun add(key: Block, suffix: String, value: String) = slave.add("${key.descriptionId}.${suffix}", value)
		fun add(key: Item, value: String) = slave.add(key, value)
		fun add(key: Item, suffix: String, value: String) = slave.add("${key.descriptionId}.${suffix}", value)
		fun add(key: ItemStack, value: String) = slave.add(key, value)
		fun add(key: Enchantment, value: String) = slave.add(key, value)
		fun add(key: MobEffect, value: String) = slave.add(key, value)
		fun add(key: EntityType<*>, value: String) = slave.add(key, value)

		fun death(key: String, value: String) = slave.add("death.attack.$key", value)
		fun stat(key: String, value: String) = slave.add("stat.${OverdriveThatMatters.MOD_ID}.$key", value)

		fun add(key: AndroidResearchType<*>, value: String) = slave.add(key.displayId, value)
		fun add(key: AndroidResearchType<*>, suffix: String, value: String) = slave.add("${key.displayId}.$suffix", value)

		fun add(key: AndroidFeatureType<*>, value: String) = slave.add(key.displayId, value)
		fun add(key: AndroidFeatureType<*>, suffix: String, value: String) = slave.add("${key.displayId}.$suffix", value)

		fun misc(key: String, value: String) = slave.add("otm.$key", value)
		fun sound(key: String, value: String) = slave.add("otm.sound.$key", value)
	}

	inner class Colors(
		language: String,

		val white: String,
		val orange: String,
		val magenta: String,
		val lightBlue: String,
		val yellow: String,
		val lime: String,
		val pink: String,
		val gray: String,
		val lightGray: String,
		val cyan: String,
		val purple: String,
		val blue: String,
		val brown: String,
		val green: String,
		val red: String,
		val black: String,
	) {
		val slave: LanguageProvider = slaves.computeIfAbsent(language, ::Slave)

		fun add(list: ColoredDecorativeBlock, toFormat: String) {
			slave.add(list.whiteBlock, toFormat.format(white))
			slave.add(list.orangeBlock, toFormat.format(orange))
			slave.add(list.magentaBlock, toFormat.format(magenta))
			slave.add(list.lightBlueBlock, toFormat.format(lightBlue))
			slave.add(list.yellowBlock, toFormat.format(yellow))
			slave.add(list.limeBlock, toFormat.format(lime))
			slave.add(list.pinkBlock, toFormat.format(pink))
			slave.add(list.grayBlock, toFormat.format(gray))
			slave.add(list.lightGrayBlock, toFormat.format(lightGray))
			slave.add(list.cyanBlock, toFormat.format(cyan))
			slave.add(list.purpleBlock, toFormat.format(purple))
			slave.add(list.blueBlock, toFormat.format(blue))
			slave.add(list.brownBlock, toFormat.format(brown))
			slave.add(list.greenBlock, toFormat.format(green))
			slave.add(list.redBlock, toFormat.format(red))
			slave.add(list.blackBlock, toFormat.format(black))
		}

		fun add(
			whiteBlock: Block,
			orangeBlock: Block,
			magentaBlock: Block,
			lightBlueBlock: Block,
			yellowBlock: Block,
			limeBlock: Block,
			pinkBlock: Block,
			grayBlock: Block,
			lightGrayBlock: Block,
			cyanBlock: Block,
			purpleBlock: Block,
			blueBlock: Block,
			brownBlock: Block,
			greenBlock: Block,
			redBlock: Block,
			blackBlock: Block,

			toFormat: String,
		) {
			slave.add(whiteBlock, toFormat.format(white))
			slave.add(orangeBlock, toFormat.format(orange))
			slave.add(magentaBlock, toFormat.format(magenta))
			slave.add(lightBlueBlock, toFormat.format(lightBlue))
			slave.add(yellowBlock, toFormat.format(yellow))
			slave.add(limeBlock, toFormat.format(lime))
			slave.add(pinkBlock, toFormat.format(pink))
			slave.add(grayBlock, toFormat.format(gray))
			slave.add(lightGrayBlock, toFormat.format(lightGray))
			slave.add(cyanBlock, toFormat.format(cyan))
			slave.add(purpleBlock, toFormat.format(purple))
			slave.add(blueBlock, toFormat.format(blue))
			slave.add(brownBlock, toFormat.format(brown))
			slave.add(greenBlock, toFormat.format(green))
			slave.add(redBlock, toFormat.format(red))
			slave.add(blackBlock, toFormat.format(black))
		}

		fun add(
			whiteItem: Item,
			orangeItem: Item,
			magentaItem: Item,
			lightBlueItem: Item,
			yellowItem: Item,
			limeItem: Item,
			pinkItem: Item,
			grayItem: Item,
			lightGrayItem: Item,
			cyanItem: Item,
			purpleItem: Item,
			blueItem: Item,
			brownItem: Item,
			greenItem: Item,
			redItem: Item,
			blackItem: Item,

			toFormat: String,
		) {
			slave.add(whiteItem, toFormat.format(white))
			slave.add(orangeItem, toFormat.format(orange))
			slave.add(magentaItem, toFormat.format(magenta))
			slave.add(lightBlueItem, toFormat.format(lightBlue))
			slave.add(yellowItem, toFormat.format(yellow))
			slave.add(limeItem, toFormat.format(lime))
			slave.add(pinkItem, toFormat.format(pink))
			slave.add(grayItem, toFormat.format(gray))
			slave.add(lightGrayItem, toFormat.format(lightGray))
			slave.add(cyanItem, toFormat.format(cyan))
			slave.add(purpleItem, toFormat.format(purple))
			slave.add(blueItem, toFormat.format(blue))
			slave.add(brownItem, toFormat.format(brown))
			slave.add(greenItem, toFormat.format(green))
			slave.add(redItem, toFormat.format(red))
			slave.add(blackItem, toFormat.format(black))
		}

		fun addBlocks(list: List<Block>, toFormat: String) {
			add(
				whiteBlock = list[0],
				orangeBlock = list[1],
				magentaBlock = list[2],
				lightBlueBlock = list[3],
				yellowBlock = list[4],
				limeBlock = list[5],
				pinkBlock = list[6],
				grayBlock = list[7],
				lightGrayBlock = list[8],
				cyanBlock = list[9],
				purpleBlock = list[10],
				blueBlock = list[11],
				brownBlock = list[12],
				greenBlock = list[13],
				redBlock = list[14],
				blackBlock = list[15],

				toFormat = toFormat
			)
		}

		fun addItems(list: List<Item>, toFormat: String) {
			add(
				whiteItem = list[0],
				orangeItem = list[1],
				magentaItem = list[2],
				lightBlueItem = list[3],
				yellowItem = list[4],
				limeItem = list[5],
				pinkItem = list[6],
				grayItem = list[7],
				lightGrayItem = list[8],
				cyanItem = list[9],
				purpleItem = list[10],
				blueItem = list[11],
				brownItem = list[12],
				greenItem = list[13],
				redItem = list[14],
				blackItem = list[15],

				toFormat = toFormat
			)
		}
	}

	val decorativeEnglish = Colors("en_us",
		"White",
		"Orange",
		"Magenta",
		"Light Blue",
		"Yellow",
		"Lime",
		"Pink",
		"Gray",
		"Light Gray",
		"Cyan",
		"Purple",
		"Blue",
		"Brown",
		"Green",
		"Red",
		"Black",
	)

	val english = Builder("en_us")
	val russian = Builder("ru_ru")

	fun getProvider(language: String): LanguageProvider = slaves.computeIfAbsent(language, ::Slave)
}
